/* All Rights reserved */

#import <AppKit/AppKit.h>
#import "TimeView.h"

@implementation TimeView

- (void) mouseDown: (NSEvent *) event
{
   NSRect titleFrame = [box titleRect];
   NSPoint windowLocation = [event locationInWindow];
   NSPoint viewLocation = [self convertPoint: windowLocation fromView: [self superview]];
   BOOL status = NSMouseInRect(viewLocation, titleFrame, NO);
   if (status == YES)
      {
         [NSBundle loadNibNamed: @"TimeZonePanel.gorm" owner: self]; 
         [zonePanel makeFirstResponder: zoneField];
         [NSApp runModalForWindow: zonePanel];
      }
}

- (void) cancelAction: (id) sender
{
   [NSApp abortModal];
   [zonePanel close];
}

- (void) okAction: (id) sender
{
   NSTimeZone *tempZone;
   tempZone = [NSTimeZone timeZoneWithName: [zoneField stringValue]];
   [NSApp stopModal];
   [zonePanel close];
   if (tempZone == nil)
      {
         NSRunAlertPanel(@"Warning!",
                         @"Wrong Time Zone !!",
                         @"OK", nil, nil);
      }
   else
      {
         [date setTimeZone: tempZone];
         [box setTitle: [tempZone description]];
         [self setDate: date];
      }
}

- (id) initWithFrame: (NSRect) frame
{
  self = [super initWithFrame: frame];

  box = [[NSBox alloc] initWithFrame: NSMakeRect(0, 0,
                                                 frame.size.width,
                                                 frame.size.height)];
  [box setBorderType: NSGrooveBorder];
  [box setTitlePosition: NSAtTop];
  [box setTitle: @"Local Time"];

  clockView = [[ClockView alloc] initWithFrame: NSMakeRect(0, 70, 
                                                          frame.size.width,
                                                          frame.size.height)];

  labelDate = [[NSTextField alloc] initWithFrame: NSMakeRect(10, 45, 35, 20)];
  [labelDate setStringValue: @"Date: "];
  [labelDate setBezeled: NO];
  [labelDate setBackgroundColor: [NSColor windowBackgroundColor]];
  [labelDate setEditable: NO];

  labelTime = [[NSTextField alloc] initWithFrame: NSMakeRect(10, 15, 35, 20)];
  [labelTime setStringValue: @"Time: "];
  [labelTime setBezeled: NO];
  [labelTime setBackgroundColor: [NSColor windowBackgroundColor]];
  [labelTime setEditable: NO];

  localDate = [[NSTextField alloc] initWithFrame: NSMakeRect(55, 45, 130, 20)];

  localTime = [[NSTextField alloc] initWithFrame: NSMakeRect(55, 15, 130, 20)];

  [box addSubview: clockView];
  [box addSubview: labelDate];
  [box addSubview: labelTime];
  [box addSubview: localDate];
  [box addSubview: localTime];
  RELEASE(clockView);
  RELEASE(labelDate);
  RELEASE(labelTime);
  RELEASE(localDate);
  RELEASE(localTime);

  [self addSubview: box];
  RELEASE(box);

  [self setDate: [NSCalendarDate date]];

  return self;
}

- (NSCalendarDate *) date
{
  return date;
}

- (void) setDate: (NSCalendarDate *) aDate
{
  ASSIGN(date, aDate);
  RETAIN(date);
  [date setTimeZone: [NSTimeZone timeZoneWithName: [box title]]];
  [date setCalendarFormat: @"%a, %b %e, %Y"];
  [localDate setStringValue: [date description]];
  [date setCalendarFormat: @"%H : %M : %S"];
  [localTime setStringValue: [date description]];
  [clockView setDate: date];
  AUTORELEASE(date);
}

@end
